import { program, Command } from '../program.ts';
import { initUser, logger, close } from './common.ts';
const usrCommand = new Command('user').description('用户相关操作');
program.addCommand(usrCommand);

const changePwd = new Command('pwd')
  .description('修改用户密码')
  .option('-u, --username <username>', '用户名')
  .option('-p, --password <password>', '新密码')
  .action(async (opts) => {
    const username = opts.username;
    const password = opts.password;
    if (!username) {
      logger.error('用户名不能为空');
      close();
      return;
    }
    const { User } = await initUser();
    const newPassword = password || 'kevisual';

    logger.info(`用户名: ${username}`);
    logger.info(`新密码: ${newPassword}`);
    const user = await User.findOne({ where: { username: username }, logging: false });
    if (!user) {
      logger.error('用户不存在');
      return;
    }
    const newP = await user.createPassword(newPassword);
    logger.info('新密码加密成功', '新密码: ', newPassword);
    close();
  });
usrCommand.addCommand(changePwd);

const list = new Command('list').description('列出所有用户').action(async () => {
  console.log('列出所有用户 start');
  const { User } = await initUser();
  console.log('列出所有用户');
  const users = await User.findAll({ limit: 10, order: [['createdAt', 'DESC']] });
  if (users.length === 0) {
    logger.info('没有用户');
    return;
  }
  users.forEach((user) => {
    console.log(`用户名: ${user.username}`);
  });
  console.log(`用户数量: ${users.length}`);
  await close();
});

usrCommand.addCommand(list);
